<?php
    
/*
***************************************
*
* PrestaShop - Open Source eCommerce Solution
* @link:               www.prestashop.com
*
* @script name:        PrestaShop TrustPay Payment
* @purpose:            Module for accepting payments by TrustPay multi-payment gateway (member of Meracrest group)
* @type:               Payment module
* @author:             Gabriel Schwardy ( addons@caleydon.com )
* @copyright:          (c) 2001-2020 Caleydon s.r.o. ( www.caleydon.com )
*
***************************************
*/

/**
 * @since 1.5.0
 */

class TrustpayPaymentModuleFrontController extends ModuleFrontController
{
    public $ssl = true;
    public $display_column_left = false;
    public $display_column_right = false;

    /**
     * @see FrontController::initContent()
     */
    public function initContent()
    {
        parent::initContent();

        /*
        * Parameter values which are different
        *   - order id vs. cart id
        *   - sign
        */
        $go_payment = Tools::getValue('process');

        /*
        * Parameter values which are different
        *   - order id vs. cart id
        *   - sign
        */
        $go_type = Tools::getValue('type');

        /*
        * Call the payment method
        *   - create an order BEFORE the payment (default)
        *   - create an order AFTER the payment
        */
        $order_method = $this->module->ORDER_METHOD;

        $paymenttype = '0';

        if (isset($go_type) && $go_type == 'trustpay') {

            $payment_type = 'trustpay';
            $payment_url = $this->module->PAYURL;
            $order_state = (int)Configuration::get('TRUSTPAY_OS_AWAITING');
            $order_private_msg = $this->module->getL('msg_pm_awaiting');

         } elseif (isset($go_type) && $go_type == 'trustcard') {

            $payment_type = 'trustcard';
            $payment_url = $this->module->PAYURLCARD;
            $order_state = (int)Configuration::get('TRUSTPAY_OS_CARDWAITING');
            $order_private_msg = $this->module->getL('msg_pm_trustcard');

        }

        $cart = $this->context->cart;

        /* Set payment currency */
        /*
        *  See module code for list of supported currencie by TrustPay
        */
/*
        $currency_code = $this->context->currency->iso_code;
        if (($currency_code == 'EUR' && $this->module->CURR == 'EUR')
            || ($currency_code == 'CZK' && $this->module->CURR == 'CZK')) {
            $currency_current = $currency_code;
        } else {
            $currency_current = 'no';
        }
*/
        if ($this->module->CURR == $this->context->currency->iso_code) {
            $currency_current = $this->context->currency->iso_code;
        } else {
            $currency_current = 'no';
        }

        $total = number_format($cart->getOrderTotal(true, 3), 2, '.', '');

        if (isset($go_payment) && $go_payment == 'pay') {
            /* Create a new order with Waiting status: get Order ID from DB */
            $customer = new Customer((int)$cart->id_customer);
            $this->module->validateOrder(
                (int)$cart->id, 
                $order_state, 
                $total, 
                $this->module->displayName, 
                $order_private_msg, 
                array(), 
                (int)$this->context->currency->id, 
                false, 
                $customer->secure_key);

            $order = new Order($this->module->currentOrder); // Just created order
            $order_id = (int)$order->id;
            $order_ref = $order->reference;
        } else {
            /* Order doen't exist yet: Order ID = Cart ID */
            $order_id = (int)($cart->id);
            $order_ref = '';
        }

        /* Create signature */
        $message = sprintf("%d/%s/%s/%s/%d", $this->module->AID, $total, $currency_current, $order_id, $paymenttype);
        $signature = $this->module->GetSignature($this->module->tp_pwd, $message);

        if ($this->module->TAX_LABEL == '1') {
            $total_to_pay = sprintf('%1$s (%2$s)', Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH)), $this->module->getL('tax_incl'));
        } else {
            $total_to_pay = Tools::displayPrice($cart->getOrderTotal(true, Cart::BOTH));
        }

        /* Payment variables */
        $this->context->smarty->assign(array(
            'module_name' => $this->module->displayName,
            'module_filename' => $this->module->name,
            'theme' => $this->module->THEME,
            'js_path_jq' => $this->module->js_path_jq,
            'js_popup' => $this->module->js_popup,
            'order_method' => $order_method,
            'nbProducts' => $cart->nbProducts(),
            'debug' => $this->module->DEBUG,
            'payment_url' => $payment_url,
            'payment_type' => $payment_type,
            'accountid' => $this->module->AID,
            'total' => $total,
            'total_to_pay' => $total_to_pay,
            'currency' => $currency_current,
            'currency_accepted' => $this->module->CURR,
            'order_id' => $order_id,
            'order_id_string' => str_pad($order_id, 9, "0", STR_PAD_LEFT),
            'paymenttype' => $paymenttype,
            'signature' => $signature,
            'localization' => $this->module->LANGUAGE,
            'email' => urlencode($this->context->cookie->email),
            'country' => $this->module->COUNTRY,
            'description' => urlencode($this->module->DESCRIPTION),
            'rurl' => urlencode($this->module->return_url),
            'curl' => urlencode($this->module->return_url),
            'eurl' => urlencode($this->module->return_url),
            'nurl' => urlencode($this->module->notif_url),
            'message' => $message,
//          'errors' => $this->module->_postErrors,
            'cart_currency' => $this->context->currency->name,
            'cart_currency_iso' => $this->context->currency->iso_code,
            'order_url' => $this->context->link->getPageLink('order', null, null, 'step=3'),
            'payment_logo' => Media::getMediaPath(_PS_MODULE_DIR_.$this->module->name.'/views/img/'.$this->module->name.'-logo.png'),
            'image_path' => Media::getMediaPath(_PS_MODULE_DIR_.$this->module->name.'/views/img/'),

            'img_alt_banks' => $this->module->getL('img_alt_banks'),
            'img_alt_tc' => $this->module->getL('img_alt_tc'),
            'img_alt_visa' => $this->module->getL('img_alt_visa'),
            'img_alt_visa_electron' => $this->module->getL('img_alt_visa_electron'),
            'img_alt_mastercard' => $this->module->getL('img_alt_mastercard'),
            'img_alt_maestro' => $this->module->getL('img_alt_maestro'),

            'logo_tp' => $this->module->logo_tp,
            'logo_tc' => $this->module->logo_tc,
            'logo_tb' => $this->module->logo_tb,
            'logo_slsp' => $this->module->logo_slsp,
            'logo_vub' => $this->module->logo_vub,
            'logo_pb' => $this->module->logo_pb,
            'logo_csob' => $this->module->logo_csob,
            'logo_ib' => $this->module->logo_ib,
            'logo_visa' => $this->module->logo_visa,
            'logo_visa_electron' => $this->module->logo_visa_electron,
            'logo_mastercard' => $this->module->logo_mastercard,
            'logo_maestro' => $this->module->logo_maestro,

        ));

        if ($currency_current == 'no') {
            $this->setTemplate('module:trustpay/views/templates/front/no_currency.tpl');
        } else {
            $this->setTemplate('module:trustpay/views/templates/front/payment_execution.tpl');
        }

        /* DOCS: 
        ** https://doc.trustpay.eu/#wire-payments
        **/
        if (isset($go_payment) && $go_payment == 'pay') {
            /* Building payment URL */
            $url = $payment_url.'?';
            $url .= 'AccountId='.$this->module->AID;
            $url .= '&Amount='.$total;
            $url .= '&Currency='.$currency_current;
            $url .= '&Reference='.$order_id;
            $url .= '&PaymentType='.$paymenttype;
            $url .= '&Signature='.$signature;
            $url .= '&ReturnUrl='.urlencode($this->module->return_url);
            $url .= '&CancelUrl='.urlencode($this->module->return_url);
            $url .= '&ErrorUrl='.urlencode($this->module->return_url);
            $url .= '&NotificationUrl='.urlencode($this->module->notif_url);
            if (isset($this->module->LANGUAGE)) {
                $url .= '&Localization='.$this->module->LANGUAGE;
            }
            if (isset($this->module->COUNTRY)) {
                $url .= '&Country='.$this->module->COUNTRY;
            }
            if (isset($this->module->DESCRIPTION)) {
                $url .= '&Description='.$this->module->DESCRIPTION;
            } else {
                $url .= '&Description=ID%20objednavky%3A%20'.$order_id;
            }
            if (isset($this->context->cookie->email)) {
                $url .= '&Email='.$this->context->cookie->email;
            }

            /* Redirect on payment gateway with payment request */
            Tools::redirect($url);
        }

    }
}
/* EOF */