<?php

/*
***************************************
*
* PrestaShop - Open Source eCommerce Solution
* @link:               www.prestashop.com
*
* @script name:        PrestaShop TrustPay Payment
* @purpose:            Module for accepting payments by TrustPay multi-payment gateway (member of Meracrest group)
* @type:               Payment module
* @author:             Gabriel Schwardy ( addons@caleydon.com )
* @copyright:          (c) 2001-2020 Caleydon s.r.o. ( www.caleydon.com )
*
***************************************
*/

/*
 * RESULT.PHP - Receiving the customer after payment and creatin a new order. 
 * DOCS TrustPay: https://doc.trustpay.eu/?php#wire-result
 * DOCS TrustCard: https://doc.trustpay.eu/?php#cards-result14
*/

include(dirname(__FILE__).'/../../../config/config.inc.php');
include(dirname(__FILE__).'/../../../init.php');
require_once(dirname(__FILE__).'/../trustpay.php');

$errors = '';
$trustpay = new Trustpay();

$debug = Configuration::get('TRUSTPAY_DEBUG');
// $debug = '1';
$order_always = Configuration::get('TRUSTPAY_ORDER_ALWAYS'); // values: always / successful_only
$order_method = Configuration::get('TRUSTPAY_ORDER_METHOD'); // values: before / after
$is_order_valid = 'no';

/* 1. BUILDING VARIABLES FROM RESULT URL */
/* IMPORTANT: DO NOT USE (int)($_GET['VALUE']) BECAUSE IT REMOVE 0 FROM THE STRING BEGINING! */

$tp_ref = Tools::getValue('Reference'); // Reference (merchant's payment identification) (cart ID) / Varchar(500)
$tp_res = Tools::getValue('ResultCode'); // Result code / Numeric(4)
$tp_arid = Tools::getValue('AcquirerResponseId'); // Card acquirer result code / Varchar(128)
$tp_prid = Tools::getValue('PaymentRequestId'); // Payment request ID / Numeric(10)

/* 2. GET OTHER VARIABLES */
$cart = new Cart((int)($tp_ref));
$total = (float)(number_format($cart->getOrderTotal(true, 3), 2, '.', ''));
$customer = new Customer((int)$cart->id_customer);

if (isset($debug) && $debug == '1') {
    echo 'Reference: '.$tp_ref.'<br>';
    echo 'ResultCode: '.$tp_res.'<br>';
    echo 'AcquirerResponseId: '.$tp_arid.'<br>';
    echo 'PaymentRequestId: '.$tp_prid.'<br>';
}

/* 3. LOAD A TRANSACTION RESULT */
/*
* Note: Only result codes 0, 3 and 4 received in a notification 
* can be treated as a successfully executed payment 
* which has been or is guaranteed to be credited 
* to merchant's account in TrustPay. 
* Docs: https://doc.trustpay.eu/#codes-result
*/

if (isset($tp_res) && $tp_res == '0') {
    /* Redirect SUCCESS */
    $order_status = 'success';
    $is_order_valid = 'yes';
}
elseif (isset($tp_res) && $tp_res == '1') {
    /* Redirect PENDING */
    $order_status = 'pending';
}
elseif (isset($tp_res) && $tp_res == '2') {
    /* Redirect ANNOUNCED */
    $order_status = 'announced';
}
elseif (isset($tp_res) && $tp_res == '3') {
    /* Redirect AUTHORIZED */
    $order_status = 'authorized';
    $is_order_valid = 'yes';
}
elseif (isset($tp_res) && $tp_res == '4') {
    /* Redirect PROCESSING */
    $order_status = 'processing';
    $is_order_valid = 'yes';
}
elseif (isset($tp_res) && $tp_res == '5') {
    /* Redirect AUTHORIZEDONLY */
    $order_status = 'authorizedonly';
}
elseif (isset($tp_res) && $tp_res == '1001') {
    /*
    1001 - INVALID REQUEST [redirect]
    Data sent is not properly formatted
    */
    $errors .= $trustpay->getL('msg_1001');
    $order_status = 'error';
}
elseif (isset($tp_res) && $tp_res == '1002') {
    /*
    1002 - UNKNOWN ACCOUNT [redirect]
    Account with specified ID was not found. 
    Please check if you are using correct account number - AID.
    */
    $errors .= $trustpay->getL('msg_1002');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1003') {
    /*
    1003 - MERCHANT ACCOUNT DISABLED [redirect]
    Merchant account has been disabled
    */
    $errors .= $trustpay->getL('msg_1003');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1004') {
    /*
    1004 - INVALID SIGN [redirect]
    The message is not signed correctly
    */
    $errors .= $trustpay->getL('msg_1004');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1005') {
    /*
    1005 - USER CANCEL [redirect]
    Customer has cancelled the payment
    */
    $errors .= $trustpay->getL('msg_1005');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1006') {
    /*
    1006 - INVALID AUTHENTICATION [redirect]
    Request was not properly authenticated
    */
    $errors .= $trustpay->getL('msg_1006');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1007') {
    /*
    1007 - DISPOSABLE BALANCE [redirect]
    Requested transaction amount is greater than disposable balance
    */
    $errors .= $trustpay->getL('msg_1007');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1008') {
    /*
    1008 - SERVICE NOT ALLOWED [redirect]
    Service cannot be used or permission to use given service has not been granted.
    If you receive current code, please contact TrustPay for more information.
    */
    $errors .= $trustpay->getL('msg_1008');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1009') {
    /*
    1009 - PAYSAFECARD TIMEOUT [redirect]
    Cards allocation will be cancelled
    */
    $errors .= $trustpay->getL('msg_1009');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1010') {
    /*
    1010 - TRANSACTION NOT FOUND [redirect]
    Transaction with specified ID was not found
    */
    $errors .= $trustpay->getL('msg_1010');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1011') {
    /*
    1011 - UNSUPPORTED TRANSACTION [redirect]
    The requested action is not supported for the transaction
    */
    $errors .= $trustpay->getL('msg_1011');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1014') {
    /*
    1014 - REJECTED TRANSACTION [client redirects]
    Transaction was rejected by issuer
    */
    $errors .= $trustpay->getL('msg_1014');
    $order_status = 'error';
} 
elseif (isset($tp_res) && $tp_res == '1100') {
    /*
    1100 - GENERAL ERROR [redirect]
    Internal error has occurred
    */
    $errors .= $trustpay->getL('msg_1100');
    $order_status = 'error';
}
elseif (isset($tp_res) && $tp_res == '1101') {
    /*
    1101 - UNSUPPORTED CURRENCY CONVERSION [redirect]
    Currency conversion for requested currencies is not supported
    */
    $errors .= $trustpay->getL('msg_1101');
    $order_status = 'error';
}
else {
    $errors .= $trustpay->getL('msg_noverif');
    $order_status = 'error';
}

/* 4. LOAD A REJECTED CARD TRANSACTION RESULT */
/*
* DOCS: https://doc.trustpay.eu/#codes-acq-result
*/

if (isset($tp_arid) && $tp_arid == '100.100.303') {
    /* Redirect SUCCESS */
//  $order_status = 'success';
//  $is_order_valid = 'yes';
}

/* 5. CREATE AN ORDER FROM CART + PROCESSING MESSAGE */
if ($is_order_valid == 'yes' AND isset($tp_ref) AND $tp_ref != '') {

    $msg = '';

    if (isset($debug) && $debug != '1') {
        /* TRUSTPAY_OS_PROCESSING */
        /* Create new order */
        $trustpay->validateOrder(
            (int)$tp_ref, 
            (int)Configuration::get('TRUSTPAY_STATUS_PROCESSING'), 
            $total, 
            $trustpay->displayName, 
            $msg, 
            array(), 
            null, 
            false, 
            $customer->secure_key
        );
    }
}

if (isset($debug) && $debug == '1') {
    echo '<br>order_status: '.$order_status.'<br>';
    echo 'is_order_valid: '.$is_order_valid.'<br><br>';
}

/* Set an Order on Payment Error status */
/*
if (isset($is_order_valid) && $is_order_valid == 'no' && isset($order_method) && $order_method == 'before') {
    $history = new OrderHistory();
    $history->id_order = (int)($tp_ref);
    $history->changeIdOrderState((int)Configuration::get('TRUSTPAY_STATUS_FAIL'), $history->id_order);
    $history->addWithemail();

    $private_message = '<br>'.$trustpay->getL('msg_paymentinfo').'<br>';
    $private_message .= $trustpay->getL('msg_error_value').' '.$errors.'<br>';
    $private_message .= $trustpay->getL('msg_ref_value').' '.$tp_ref.'<br>';
    $private_message .= $trustpay->getL('msg_res_value').' '.$tp_res.'<br>';
    if (isset($tp_prid) && !empty($tp_prid)) {
        $private_message .= $trustpay->getL('msg_prid_value').' '.$tp_prid.'<br>';
    }

    // Add the private order message to an order
    $trustpay->_addNewPrivateMessage((int)$tp_ref, $private_message);
}
*/

/* 6. REDIRECT */
$link = Context::getContext()->link->getModuleLink('trustpay', 'result', array('cart' => $tp_ref, 'prid' => $tp_prid, 'status' => $order_status));

if (isset($debug) && $debug != '1') {
    Tools::redirect($link);
} else {
    echo 'Redirect link: '.$link;
}

/* EOF */